import { generateUuid } from '../generateUuid.js';
import { NodeTypeRegistry } from '../Nodes/NodeTypeRegistry.js';
import { OnCustomEvent } from '../Profiles/Core/CustomEvents/OnCustomEvent.js';
import { TriggerCustomEvent } from '../Profiles/Core/CustomEvents/TriggerCustomEvent.js';
import { VariableGet } from '../Profiles/Core/Variables/VariableGet.js';
import { VariableSet } from '../Profiles/Core/Variables/VariableSet.js';
export class Graph {
    constructor(registry) {
        this.registry = registry;
        this.name = '';
        this.nodes = {};
        this.variables = {};
        this.customEvents = {};
        this.metadata = {};
        this.dynamicNodeRegistry = new NodeTypeRegistry();
        this.version = 0;
    }
    updateDynamicNodeDescriptions() {
        this.dynamicNodeRegistry.clear();
        Object.keys(this.variables).forEach((variableId) => {
            this.dynamicNodeRegistry.register(VariableGet.GetDescription(this, variableId), VariableSet.GetDescription(this, variableId));
        });
        Object.keys(this.customEvents).forEach((customEventId) => {
            this.dynamicNodeRegistry.register(OnCustomEvent.GetDescription(this, customEventId), TriggerCustomEvent.GetDescription(this, customEventId));
        });
    }
    createNode(nodeTypeName, nodeId = generateUuid()) {
        if (nodeId in this.nodes) {
            throw new Error(`can not create new node of type ${nodeTypeName} with id ${nodeId} as one with that id already exists.`);
        }
        let nodeDescription = undefined;
        if (this.registry.nodes.contains(nodeTypeName)) {
            nodeDescription = this.registry.nodes.get(nodeTypeName);
        }
        if (this.dynamicNodeRegistry.contains(nodeTypeName)) {
            nodeDescription = this.dynamicNodeRegistry.get(nodeTypeName);
        }
        if (nodeDescription === undefined) {
            throw new Error(`no registered node descriptions with the typeName ${nodeTypeName}`);
        }
        const node = nodeDescription.factory(nodeDescription, this);
        node.id = nodeId;
        this.nodes[nodeId] = node;
        return node;
    }
}
//# sourceMappingURL=Graph.js.map