import { Node } from '../../../Nodes/Node.js';
import { NodeDescription } from '../../../Nodes/NodeDescription.js';
import { Socket } from '../../../Sockets/Socket.js';
export class OnCustomEvent extends Node {
    constructor(description, graph, customEvent) {
        super(description, graph, [], [
            new Socket('flow', 'flow'),
            ...customEvent.parameters.map((parameter) => new Socket(parameter.valueTypeName, parameter.name, parameter.value, parameter.label))
        ], (context) => {
            customEvent.eventEmitter.addListener((parameters) => {
                customEvent.parameters.forEach((parameterSocket) => {
                    if (!(parameterSocket.name in parameters)) {
                        throw new Error(`parameters of custom event do not align with parameters of custom event node, missing ${parameterSocket.name}`);
                    }
                    context.writeOutput(parameterSocket.name, parameters[parameterSocket.name]);
                });
                context.commit('flow');
            });
        });
        this.customEvent = customEvent;
        this.evaluateOnStartup = true;
        this.async = true;
        this.interruptibleAsync = true;
    }
    static GetDescription(graph, customEventId) {
        const customEvent = graph.customEvents[customEventId];
        return new NodeDescription(`customEvent/onTriggered/${customEvent.id}`, 'Event', `On ${customEvent.name}`, (description, graph) => new OnCustomEvent(description, graph, customEvent));
    }
}
//# sourceMappingURL=OnCustomEvent.js.map