import { Node } from '../../../Nodes/Node.js';
import { NodeDescription } from '../../../Nodes/NodeDescription.js';
import { Socket } from '../../../Sockets/Socket.js';
export class LifecycleOnEnd extends Node {
    constructor(description, graph) {
        super(description, graph, [], [new Socket('flow', 'flow')], (context) => {
            const onEndEvent = () => {
                context.commit('flow');
            };
            const lifecycleEvents = context.graph.registry.abstractions.get('ILifecycleEventEmitter');
            lifecycleEvents.endEvent.addListener(onEndEvent);
            context.onAsyncCancelled.addListener(() => {
                lifecycleEvents.endEvent.removeListener(onEndEvent);
            });
        });
        this.async = true;
        this.evaluateOnStartup = true;
        this.interruptibleAsync = true;
    }
}
LifecycleOnEnd.Description = new NodeDescription('lifecycle/onEnd', 'Event', 'On End', (description, graph) => new LifecycleOnEnd(description, graph));
//# sourceMappingURL=LifecycleOnEnd.js.map