import { Node } from '../../../Nodes/Node.js';
import { NodeDescription } from '../../../Nodes/NodeDescription.js';
import { Socket } from '../../../Sockets/Socket.js';
export class LifecycleOnTick extends Node {
    constructor(description, graph) {
        super(description, graph, [], [
            new Socket('flow', 'flow'),
            new Socket('float', 'deltaSeconds'),
            new Socket('float', 'time')
        ], (context) => {
            let lastTickTime = Date.now();
            const onTickEvent = () => {
                const currentTime = Date.now();
                const deltaSeconds = (currentTime - lastTickTime) * 0.001;
                context.writeOutput('deltaSeconds', deltaSeconds);
                context.writeOutput('time', Date.now());
                context.commit('flow');
                lastTickTime = currentTime;
            };
            const lifecycleEvents = context.graph.registry.abstractions.get('ILifecycleEventEmitter');
            lifecycleEvents.tickEvent.addListener(onTickEvent);
            context.onAsyncCancelled.addListener(() => {
                lifecycleEvents.tickEvent.removeListener(onTickEvent);
            });
        });
        this.async = true;
        this.evaluateOnStartup = true;
        this.interruptibleAsync = true;
    }
}
LifecycleOnTick.Description = new NodeDescription('lifecycle/onTick', 'Event', 'On Tick', (description, graph) => new LifecycleOnTick(description, graph));
//# sourceMappingURL=LifecycleOnTick.js.map